"""
fritz.train
~~~~~~~~~~~

:copyright: © 2018 by Fritz Labs Incorporated
:license: MIT, see LICENSE for more details.
"""

from fritz.train.model_checkpoint import FritzSnapshotCallback

assert FritzSnapshotCallback
